#ifndef KPARANOIDLINE_H
#define KPARANOIDLINE_H

#include <klineedit.h>

class KParanoidLine : public KLineEdit
{
  TQ_OBJECT
  
public:
	KParanoidLine(TQWidget *parent=0, const char *name=0);
	~KParanoidLine();

signals:
	void backspacePressed();
	void textEntered(const TQString &);

protected:
	void keyPressEvent( TQKeyEvent *e );
};

#endif // KPARANOIDLINE_H
