#ifndef STACKLEVEL_H
#define STACKLEVEL_H

class TQLineEdit;
class TQPushButton;
class TQWidget;

#include <tqhbox.h>

class StackLevel : public TQHBox
{
	TQ_OBJECT

public:
	StackLevel(int, TQWidget *parent=0, const char *name=0);
	virtual void setEnabled(bool enabled);
	TQLineEdit *edit() { return m_edit; }
	TQPushButton *button() { return m_button; }
	
signals:
	void buttonPressed(int);
	void returnPressed(int);

protected slots:
	void emitButton();
	void emitEdit();

private:
	TQLineEdit *m_edit;
	TQPushButton *m_button;
	int id;
};

#endif
