<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="Mathemagics" version="7">

<MenuBar>

<Menu name="other"><text>&amp;Operations</text>
	<Action name="formulae" />
	<Separator />
	<Action name="+/-" />
	<Separator />

	<Menu name="trig"><text>&amp;Trig</text>
		<Action name="sin" />
		<Action name="cos" />
		<Action name="tan" />
	</Menu>

	<Menu name="bitwise"><text>&amp;Bitwise</text>
		<Action name="bitwise_and" />
		<Action name="bitwise_or" />
		<Action name="bitwise_xor" />
		<Action name="bitwise_comose" />
		<Action name="bitwise_lsh" />
		<Action name="bitwise_rsh" />
	</Menu>

	<Separator />

	<Action name="factorial" />
	<Action name="1/x" />
	<Action name="10^x" />
	<Action name="e^x" />
	<Action name="modulo" />
	<Action name="percent" />
	<Action name="sqrt" />
	<Action name="x^2" />
	<Action name="y^x" />
	<Action name="ln" />
	<Action name="log10" />
</Menu>

<Menu name="edit"><text>&amp;Edit</text>
	<Action name="revert" />
	<Separator />
	<Action name="drop" />
</Menu>

<Menu name="settings"><text>&amp;Settings</text>
	<Action name="inverse" />
	<Action name="hyperbolic" />
	<Separator />
	<Action name="base" />
	<Action name="angle" />
	<Separator />
	<Action name="keypad" />
	<Separator />
	<Merge name="StandardToolBarMenuHandler" />
</Menu>

</MenuBar>

<ToolBar name="mainToolBar" noEdit="true"><text>Main Toolbar</text>
	<Action name="lineedit" />
</ToolBar>

<ToolBar name="historyToolBar"><text>History Toolbar</text>
	<Action name="history" />
</ToolBar>

<ToolBar name="opToolBar" iconText="textonly" position="left"><text>Operations Toolbar</text>
	<Action name="+/-" />
	<Action name="1/x" />
	<Action name="e^x" />
	<Action name="percent" />
	<Action name="sqrt" />
	<Action name="x^2" />
	<Action name="y^x" />
	<Action name="ln" />
</ToolBar>

<ToolBar name="trigToolBar" iconText="textonly" position="left"><text>Trigonometry Toolbar</text>
	<Action name="sin" />
	<Action name="cos" />
	<Action name="tan" />
	<Separator />
	<Action name="hyperbolic" />
</ToolBar>

<ToolBar name="settingsToolBar" fullWidth="false" iconText="textonly"><text>Settings Toolbar</text>
	<Action name="inverse" />
	<Action name="base" />
	<Action name="angle" />
</ToolBar>

</kpartgui>
