#include <tqkeycode.h>
#include <tqstring.h>

#include <kdebug.h>

#include "kparanoidline.h"

KParanoidLine::KParanoidLine(TQWidget *parent, const char *name)
  : KLineEdit(parent, name)
{
}

KParanoidLine::~KParanoidLine()
{
}

void KParanoidLine::keyPressEvent(TQKeyEvent *e)
{
	if (e->key() == Key_BackSpace)
		emit backspacePressed();

	emit textEntered(e->text());

	TQLineEdit::keyPressEvent(e);
}

#include "kparanoidline.moc"
